import libmultilabel.linear as linear
import time
import numpy as np
import scipy.sparse as sparse
import argparse
import pickle
import os
import time
import math
from tqdm import tqdm

parser = argparse.ArgumentParser()

parser.add_argument('--seed', type=int, default=27)
parser.add_argument('--K', type=int, default=10)
parser.add_argument('--beam_width', type=int, default=10000)
parser.add_argument('--datapath', type=str, default="")

ARGS = parser.parse_args()

print("start", flush=True)
start = time.time()
with open(ARGS.datapath + '.pkl', "rb") as F:
    datasets = pickle.load(F)
print("data loading cost:", time.time()-start, flush=True)

def metrics_in_batches(model_name, batch_size):
    num_instances = datasets["test"]["x"].shape[0]
    num_batches = math.ceil(num_instances / batch_size)

    metrics = linear.get_metrics(["P@1", "P@3", "P@5"], num_classes=datasets["test"]["y"].shape[1])
    for i in range(num_batches):
        print("process batches id:", i, flush=True)
        tmp_data = datasets["test"]["x"][i * batch_size : (i + 1) * batch_size]
        total_preds = np.zeros([tmp_data.shape[0], datasets["train"]["y"].shape[1]], order='F')

        submodel_name = "./models/" + model_name 
        with open(submodel_name, "rb") as F:
            model, _= pickle.load(F)

        preds = model.predict_values(tmp_data, beam_width=ARGS.beam_width)
        total_preds += preds

        target = datasets["test"]["y"][i * batch_size : (i + 1) * batch_size].toarray()
        metrics.update(total_preds, target)

    return metrics.compute()

model_name = "Tree-with-all-labels_{data}_seed={seed}_K={K}.model".format(
        seed = ARGS.seed,
        K = ARGS.K,
        data = os.path.basename(ARGS.datapath)
        )

metrics = metrics_in_batches(model_name, 1000)

print("mean in subsampled labels:", metrics)
print("Total time:", time.time()-start)
